@@ -1,3 +1,4 @@
+import pickle
 from typing import Any, Dict, List, Optional, Union
 
 import pytest
@@ -518,6 +519,24 @@ def test_mixed_discriminated_union(data):
     assert Model(**data).model_dump() == data
 
 
+def test_list_rootmodel():
+    class A(BaseModel):
+        type: Literal['a']
+        a: str
+
+    class B(BaseModel):
+        type: Literal['b']
+        b: str
+
+    class D(RootModel[Annotated[Union[A, B], Field(discriminator='type')]]):
+        pass
+
+    LD = RootModel[List[D]]
+
+    obj = LD.model_validate([{'type': 'a', 'a': 'a'}, {'type': 'b', 'b': 'b'}])
+    assert obj.model_dump() == [{'type': 'a', 'a': 'a'}, {'type': 'b', 'b': 'b'}]
+
+
 def test_root_and_data_error():
     class BModel(BaseModel):
         value: int
@@ -530,3 +549,15 @@ def test_root_and_data_error():
         match='"RootModel.__init__" accepts either a single positional argument or arbitrary keyword arguments',
     ):
         Model({'value': 42}, other_value='abc')
+
+
+def test_pickle_root_model(create_module):
+    @create_module
+    def module():
+        from pydantic import RootModel
+
+        class MyRootModel(RootModel[str]):
+            pass
+
+    MyRootModel = module.MyRootModel
+    assert MyRootModel(root='abc') == pickle.loads(pickle.dumps(MyRootModel(root='abc')))
